"""Dummy models to be used in test cases of the ``generic_positions`` app."""
from django.contrib.contenttypes import generic
from django.db import models


class DummyModel(models.Model):
    """Dummy to be used in test cases of the ``generic_positions`` app."""
    name = models.CharField(max_length=256, blank=True)

    def __unicode__(self):
        return self.name


# We are just monkeypatching here for testing purposes. Of course you would add
# the GenericRelation directly to your model if it is part of your own app.
# The monkeypatching will only be used for models of third party apps.
DummyModel.add_to_class(
    'generic_position',
    generic.GenericRelation('generic_positions.ObjectPosition'),
)



"""URLs for the ``generic_positions`` app."""
from django.conf.urls import patterns, url

from .views import PositionBulkUpdateView


urlpatterns = patterns(
    '',
    url(r'update/$',
        PositionBulkUpdateView.as_view(),
        name='position_bulk_update'),
)



"""Views for the ``generic_positions`` app."""
from django.http import HttpResponse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.csrf import requires_csrf_token
from django.views.generic import View

from .models import save_positions


class PositionBulkUpdateView(View):
    """View to update position objects with a POST dictionary."""
    http_method_names = [u'post', ]

    def post(self, request, *args, **kwargs):
        if request.is_ajax():
            save_positions(request.POST)
        return HttpResponse('done')



"""Tests for the template tags of the ``generic_positions`` app."""
from django.contrib.admin.sites import AdminSite
from django.contrib.admin.views.main import ChangeList
from django.template import Context, Template
from django.test import TestCase
from django.test.client import RequestFactory

from ..admin import GenericPositionsAdmin
from ..templatetags.position_tags import order_by_position
from .factories import DummyModelFactory
from .test_app.models import DummyModel


class PositionResultListTestCase(TestCase):
    """Tests for the ``position_result_list`` tag."""
    longMessage = True

    def setUp(self):
        self.first_model = DummyModelFactory()
        DummyModelFactory()
        DummyModelFactory()

    def test_render_tag(self):
        request = RequestFactory().get('/')
        t = Template('{% load position_tags %}{% position_result_list cl %}')
        change_list = ChangeList(
            request=request, model=DummyModel, list_display=['name'],
            list_display_links=None, list_filter=None, date_hierarchy=None,
            search_fields=None, list_select_related=None, list_per_page=100,
            list_max_show_all=None, list_editable=None,
            model_admin=GenericPositionsAdmin(DummyModel, AdminSite()))
        change_list.formset = None
        c = Context({'cl': change_list})
        self.assertIn('name="position-{0}"'.format(self.first_model.id),
                      t.render(c))


class PositionInputTestCase(TestCase):
    """Tests for the ``position_input`` tag."""
    longMessage = True

    def setUp(self):
        self.first_model = DummyModelFactory()

    def test_render_tag(self):
        t = Template('{% load position_tags %}{% position_input obj %}')
        c = Context({'obj': self.first_model})
        self.assertIn('name="position-{0}"'.format(self.first_model.id),
                      t.render(c))


class OrderByPositionTestCase(TestCase):
    """Tests for the ``order_by_position`` filter."""
    longMessage = True

    def setUp(self):
        self.first_model = DummyModelFactory()
        DummyModelFactory()
        self.last_model = DummyModelFactory()

    def test_tag(self):
        qs = DummyModel.objects.all()
        self.assertEqual(qs[0].name, self.first_model.name)
        qs = order_by_position(qs)
        self.assertEqual(qs[0].name, self.first_model.name)
        qs = order_by_position(qs, reverse='reverse')
        self.assertEqual(qs[0].name, self.last_model.name)



"""Models for the ``generic_positions`` app."""
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.db import models
from django.utils.translation import ugettext_lazy as _


class ObjectPosition(models.Model):
    """
    Model to add a position field to any kind of object.

    :content_object: Object, which now has a position field.
    :position: Current position integer of the object.

    """
    # Generic Foreign Key Bundle
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey()

    # Other fields
    position = models.PositiveIntegerField(
        verbose_name=_('Position'),
        null=True, blank=True,
    )

    class Meta:
        ordering = ('position', )


def save_positions(post_data, queryset=None):
    """
    Function to update a queryset of position objects with a post data dict.

    :post_data: Typical post data dictionary like ``request.POST``, which
      contains the keys of the position inputs.
    :queryset: Queryset of the model ``ObjectPosition``.

    """
    if not queryset:
        queryset = ObjectPosition.objects.all()
    for key in post_data:
        if key.startswith('position-'):
            try:
                obj_id = int(key.replace('position-', ''))
            except ValueError:
                continue
            queryset.filter(pk=obj_id).update(position=post_data[key])



"""Admin classes for the ``test_app`` app."""
from django.contrib import admin

from ...admin import GenericPositionsAdmin
from .models import DummyModel


admin.site.register(DummyModel, GenericPositionsAdmin)



"""Tests for the models of the ``generic_positions`` app."""
from django.test import TestCase

from ..models import ObjectPosition, save_positions
from .factories import ObjectPositionFactory


class ObjectPositionTestCase(TestCase):
    """Tests for the ``ObjectPosition`` model class."""
    longMessage = True

    def test_instantiation(self):
        """Test instatiation of the ``ObjectPosition`` model."""
        object_position = ObjectPosition()
        self.assertTrue(object_position)

    def test_save_positions_function(self):
        """Test the ``save_positions`` function."""
        object_position = ObjectPositionFactory()
        object_position2 = ObjectPositionFactory()
        post_data = {
            'position-{0}'.format(object_position.id): '2',
            'position-invalid': '2',
        }
        save_positions(post_data)

        # The obj in our dict should be updated...
        self.assertEqual(
            ObjectPosition.objects.get(id=object_position.id).position, 2)

        # ..the other one should remain the same.
        self.assertEqual(
            ObjectPosition.objects.get(id=object_position2.id).position,
            object_position2.position)



"""Factories of the ``generic_positions`` app."""
import factory

from ..models import ObjectPosition
from .test_app.models import DummyModel


class DummyModelFactory(factory.Factory):
    """Factory for the ``DummyModel`` model."""
    FACTORY_FOR = DummyModel

    name = 'Foobar'


class ObjectPositionFactory(factory.Factory):
    """Factory for the ``ObjectPosition`` model."""
    FACTORY_FOR = ObjectPosition

    content_object = factory.SubFactory(DummyModelFactory)



"""Admin classes for the ``generic_positions`` app."""
from django.contrib import admin
from django.contrib.admin.options import csrf_protect_m

from .models import save_positions


class GenericPositionsAdmin(admin.ModelAdmin):
    """Admin to let models be dragged & dropped with jQuery UI sortable."""
    change_list_template = 'generic_positions/admin/change_list.html'
    ordering = ('generic_position__position', )

    class Media:
        js = (
            '//ajax.googleapis.com/ajax/libs/jquery/1.9.1/jquery.min.js',
            '//ajax.googleapis.com/ajax/libs/jqueryui/1.10.2/jquery-ui.min.js',
            'generic_positions/js/reorder.js',
        )

    @csrf_protect_m
    def changelist_view(self, request, **kwargs):
        resp = super(GenericPositionsAdmin, self).changelist_view(
            request, **kwargs)
        if request.method == "POST" and request.is_ajax():
            save_positions(request.POST)
        return resp



"""Template tags of the ``generic_positions`` app."""
from django.contrib.admin.templatetags.admin_list import result_list
from django.contrib.contenttypes.models import ContentType
from django.template import Library
from django.utils.safestring import mark_safe

from ..models import ObjectPosition

register = Library()


@register.filter()
def order_by_position(qs, reverse=False):
    """Template filter to return a position-ordered queryset."""
    if reverse:
        return qs.order_by('-generic_position__position')
    return qs.order_by('generic_position__position')


@register.inclusion_tag('generic_positions/position.html')
def position_input(obj, visible=False):
    """Template tag to return an input field for the position of the object."""
    if not obj.generic_position.all():
        ObjectPosition.objects.create(content_object=obj)
    return {'obj': obj, 'visible': visible,
            'object_position': obj.generic_position.all()[0]}


@register.inclusion_tag('admin/change_list_results.html')
def position_result_list(cl):
    """
    Returns a template which iters through the models and appends a new
    position column.

    """
    result = result_list(cl)
    # Remove sortable attributes
    for x in range(0, len(result['result_headers'])):
        result['result_headers'][x]['sorted'] = False
        if result['result_headers'][x]['sortable']:
            result['result_headers'][x]['class_attrib'] = mark_safe(
                ' class="sortable"')
    # Append position <th> element
    result['result_headers'].append({
        'url_remove': '?o=',
        'sort_priority': 1,
        'sortable': True,
        'class_attrib': mark_safe(' class="sortable sorted ascending"'),
        'sorted': True,
        'text': 'position',
        'ascending': True,
        'url_primary': '?o=-1',
        'url_toggle': '?o=-1',
    })
    # Append the editable field to every result item
    for x in range(0, len(result['results'])):
        obj = cl.result_list[x]
        c_type = ContentType.objects.get_for_model(obj)
        # Get or create position object
        try:
            object_position = ObjectPosition.objects.get(
                content_type__pk=c_type.id, object_id=obj.id)
        except ObjectPosition.DoesNotExist:
            object_position = ObjectPosition.objects.create(content_object=obj)
        # Add the <td>
        html = ('<td><input class="vTextField" id="id_position-{0}"'
                ' maxlength="10" name="position-{0}" type="text"'
                ' value="{1}" /></td>').format(object_position.id,
                                               object_position.position)
        result['results'][x].append(mark_safe(html))
    return result



"""Tests for the views of the ``generic_positions`` app."""
from django.test import TestCase

from django_libs.tests.mixins import ViewTestMixin


class PositionBulkUpdateViewTestCase(ViewTestMixin, TestCase):
    """Tests for the ``PositionBulkUpdateView`` generic based view."""
    longMessage = True

    def get_view_name(self):
        return 'position_bulk_update'

    def test_view(self):
        self.is_callable(method='post')
        resp = self.client.post(self.get_url(),
                                HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(resp.status_code, 200)



import os
from setuptools import setup, find_packages
import generic_positions as app


def read(fname):
    try:
        return open(os.path.join(os.path.dirname(__file__), fname)).read()
    except IOError:
        return ''


setup(
    name="django-generic-positions",
    version=app.__version__,
    description=read('DESCRIPTION'),
    long_description=read('README.rst'),
    license='The MIT License',
    platforms=['OS Independent'],
    keywords='django, reusable, position, reorder, order',
    author='Tobias Lorenz',
    author_email='tobias.lorenz@bitmazk.com',
    url="https://github.com/bitmazk/django-generic-positions",
    packages=find_packages(),
    include_package_data=True,
)
